
/*************************************************************
               COVERINGS BY PARALLELOGRAMS
           Authors: P. Shmerkin and B. Solomyak
                 Revised: June 20, 2005
*************************************************************/

// This program verifies the covering assertion made in Lemma 4.2 of the paper.

#include <iostream>

using namespace std;

typedef double real;  

inline real min(real a, real b) { return a<b?a:b; }
inline real max(real a, real b) { return a>b?a:b; }

/*********************************************************************
                GENERAL LINEAR ALGEBRA OBJECTS
**********************************************************************/

//We implement some basic linear algebra objects that we will need.
//Implementation is optimized for clarity, not efficiency.

class Point {

public:

    real x;
    real y;
    
    Point() { x=y=0; }
    Point(real X, real Y) { x=X;y=Y; }
    
    inline Point operator + (Point P) 
        { return Point(x+P.x,y+P.y); }
        
    inline Point operator * (real t)
        { return Point(x*t, y*t); }
        
    //for debugging purposes
    inline void output() { cout << "[" << x << ", " << y << "]"; }

};

class Matrix {

public:

    real a;
    real b;
    real c;
    real d;
    
    Matrix() { a=b=c=d=0; }
    Matrix(real A,real B,real C,real D) { a=A; b=B; c=C; d=D; }

    inline Point operator *(Point P) 
        { return Point(a * P.x + b * P.y, c * P.x + d * P.y); }
    
    inline Matrix operator * (Matrix M)
        { return Matrix(a*M.a+b*M.c, a*M.b+b*M.d, c*M.a+d*M.c, c*M.b+d*M.d);  }
                
    inline Matrix operator * (real t)
        { return Matrix(a*t, b*t, c*t, d*t); }
        
    inline real det() { return a*d-b*c; }
    
    inline Matrix inverse() { return Matrix(d,-b,-c,a) * (1/det()); }
                
    inline Matrix power(int n) {
        if (n==0) return Matrix(1,0,0,1);
        if (n>=1) return this->power(n-1) * *this;
        if (n<0) return this->power(-n).inverse();
    }
    
    //for debugging purposes
    inline void output() { cout << "[" << a << ", " << b << " | " << c << ", " << d << "]"; }

};


/*********************************************************************
                     SPECIFIC DEFINITIONS
**********************************************************************/

//In this section we define the appropriate parameters for the problem under study.
//By modifying the relevant parameters and recompiling it is possible to check different
//coverings. For example, by modifying the eigenvalues of the matrix T (and possibly increasing
//N_ITER and N_COVER) one can find other interior points of the set of double zeros.

#define N_ITER  5             // How many iterations of the i.f.s.
#define N_COVER  243          // How many parallelograms in the covering. Must be equal to 3^N_ITER

real grid_size = 0.04;            // The size of the grid
real sq_size = 1-grid_size;      

//Matrix representing the canonical square

Matrix S(1,1,1,-1);

//Matrix representing the parallelogram U (which we want to cover)

Matrix U(-2.1, 4.9, 0.2, 2.45);

//Matrix representing the parallelogram V (whose iterates will compose the covering)

Matrix V = U * 0.95;

//The matrix used in the i.f.s.

Matrix T(pow(2.,-0.5), .7, 0, pow(2.,-0.5));

//The translation vector for the i.f.s.

Point b(1,1);

//Change of coordinates which converts T^5(V) into a canonical square S.
//S has center at the origin, side 2 and sides parallel to the axes.

Matrix A = S * V.inverse() * T.power(-5);

//The parallelogram we want to cover in the new coordinate system.

Matrix U2 = A*U;

//The vertices of this parallelogram.
//Note: the segments vert[3]-vert[0] and vert[0]-vert[1] MUST correspond
//to the LEFT sides of the parallelogram.

Point vert[4]; 

void setVertices() {

    vert[0] = Point(U2.a,U2.c);  // ~(8.12,8.50)
    vert[1] = Point(U2.b,U2.d);  // ~(32.48,25.29)
    vert[2] = vert[0]*(-1);      // ~(-8.12,-8.50)
    vert[3] = vert[1]*(-1);      // ~(-32.48,-25.29)

}

//An array containing the centers of the squares in the covering in the new coordinate system

Point covering[N_COVER];


/*********************************************************************
                       GLOBAL ROUTINES
**********************************************************************/

//Compute the centers of the covering squares

void fillCovering() {
    
    for (int i=0; i<N_COVER; i++) {
    //the ternary digits of i correspond to a word (u_1,...,u_N) with coefficients -1,0,1.
        Point q;  // q=(0,0)
        int k = i, u;
        for (int j=0; j<N_ITER; j++)  {
          u = (k%3)-1;      // get the next entry in the word
          q = T*q + b*u;    // apply the i.f.s. to q  
          k = k/3;          // move to the next digit
        }
        covering[i] = A*q;  // convert to the new coordinate system
    }
}

//Finds the x-coordinate of intersection of the line through p and q, and the horizontal line y=b.

real horizIntersect(Point p, Point q, real b) {
    real m =(q.y-p.y)/(q.x-p.x);
    return p.x + (b-p.y)/m;
}

//Returns the leftmost point in the parallelogram U2 with y-coordinate equal to h.

real leftPoint(real h) {
    return max(horizIntersect(vert[3],vert[0],h),horizIntersect(vert[0],vert[1],h));
}

//Returns the rightmost point in the parallelogram U2 with y-coordinate equal to h.

real rightPoint(real h) {
    return -leftPoint(-h);
}

//Checks whether the square with center p and half-side sq_size lies inside the i-th square in the covering.

bool checkPointInSquare(Point p, int i) {
    if (p.x <= covering[i].x - sq_size) return 0;
    if (p.y <= covering[i].y - sq_size) return 0;
    if (p.x >= covering[i].x + sq_size) return 0;
    if (p.y >= covering[i].y + sq_size) return 0;
    return 1;
}

//Checks whether the point p is inside one of the squares in the covering.

bool checkPoint(Point p) {
    for (int i=0; i<N_COVER; i++) {
        if (checkPointInSquare(p,i)) return 1;
    }
    return 0;
}

//This is the main routine. The parallelogram U2 is covered by small squares
//of half-side grid_size, and the number of those squares contained in some of the large squares
//in the covering is reported.

//Notice that the sharp step in the for statements is 2*grid_size, but we use a redundant
//one to account for possible floating point errors (which in this case are clearly negligible
//in relation to the redundancy).

bool checkCovering() {
    real max_h = max(vert[0].y,vert[1].y);  // top point of the parallelogram
    int count_covered=0, count_all=0;
    for (real y=-max_h; y<=max_h; y+=grid_size) {
        real leftLimit = min(leftPoint(y),min(leftPoint(y-grid_size),leftPoint(y+grid_size)));
        real rightLimit = min(rightPoint(y),min(rightPoint(y-grid_size),rightPoint(y+grid_size)));
        for (real x=leftLimit; x<=rightLimit; x+=grid_size) {
                count_all ++;
                count_covered += checkPoint(Point(x,y));
        }
     }          
    cout << "A total of " << count_covered << " out of " << count_all << " small squares in the grid have been covered.";
}

//The main routine simply calls the required functions fillCovering() and setVertices()
//and then checks whether the grid is covered.

int main() {

    fillCovering();
    setVertices();  
    checkCovering(); 
    
    cout << "\nPress any key and then <enter> to exit... ";
    char *exitKey;
    cin >> exitKey;
    
    return 0;

}
    
