

/*************************************************************
               DOUBLE ZEROS OF POWER SERIES
           Authors: P. Shmerkin and B. Solomyak
                 Revised: June 20, 2005
*************************************************************/

#include <iostream>
#include <fstream>
#include <list>
#include <iterator>

//An upper bound for the theoretical floating point error in the calculations.
//This is a conservative estimate, and it is still very small in relation to the quantities involved.
//We underline that all the results can be checked using exact arithmetic, at the price of the slower performance.
#define F_P_ERROR 1.0e-14
//The number of coefficients to be analyzed. In the main example it is 3 (-1, 0 and 1).
#define NDIGITS 3
//The number of derivatives to consider.In the main example it is 2 (for studying double roots). 
//Only up to 4 derivatives have been implemented, with the case of 4 derivatives being extremely slow.
#define MAX_DER 2
//The maximum number of iterations to be executed.
#define MAX_DEPTH 100

/*********************************************************************
                            DECLARATIONS
**********************************************************************/

using namespace std;

//An index representing a coefficient
typedef short int SYMBOL;
//A list of smybols; i.e. a polynomial in the class under study.
typedef list<SYMBOL> ADDRESS;
typedef ADDRESS::iterator ADDRESS_IT;

struct AddressInfo {

    ADDRESS addr;
    bool root;
    
};

//This is our main example, but it can be modified to cover other cases.
//The smallest (in absolute value) nonzero element must be 1, and 0 must be a digit as well.
//The only other example considered in the paper corresponds to digit[] = {1,-1,2,-2,0}.
long double digit[NDIGITS] = {1,-1, 0};
//The maximum digit in absolute value.
long double maxDigit = 1;

//A quick implementation of combinatorial numbers
long double C(long double k, int j) {

    long double ret = k;
    for (long double i=k-1; i > k-j ; i--) ret *= i;
    
    return ret;

}

//A class used to deal with intervals on which the existence of roots is tested
class Interval {

    long double x;
    long double y;
    
public:

    Interval() {}
    
    Interval(long double start, long double end) {
        x = start;
        y = end;
    }
    
    long double leftPoint() const { return x; }
    long double rightPoint() const { return y; }
    long double length() const { return y-x; }
    long double center() const {  return 0.5 * (y+x); }
    
    inline Interval leftHalf();
    inline Interval rightHalf(); 

};

inline Interval Interval::leftHalf() { return Interval(x, center()); }
inline Interval Interval::rightHalf() { return Interval(center(),y) ; }

//A class that encompasses the information needed to search for roots
class RootImplementation;

//A class that performs the actual algorithms to test for roots
class RootInvestigator{

    long double t;                // 1/value being evaluated   
    long double l;                // length of the interval under consideration
    int depth;
    
    friend class RootImplementation;

    inline long double f_derivativeBound(int n, int j) const;
    inline long double f_tailBound(int n, int j) const;
    
    long double compValue[MAX_DEPTH][MAX_DER];

    //Used only for the 4th derivative    
    inline long double F4(long double, long double) const;  
    
public:
    
    inline RootInvestigator(long double lambda, long double length, int maxDepth);
    
    inline bool mayContainRoot();
    inline AddressInfo getAddress();
      
};


class RootImplementation{

    int n;
    long double value[MAX_DER];
    RootInvestigator* RI;
    
    long double t() { return RI->t; }

public:

    inline RootImplementation(RootInvestigator*);
    inline RootImplementation(RootImplementation& parent, SYMBOL nextDigit);
    
    inline RootImplementation child(SYMBOL nextDigitIndex);
    
    inline bool mayContainRoot();
    bool mayContainRoot(int depth);
    
    AddressInfo getAddress(int depth, ADDRESS& l);

};

/*********************************************************************
                            IMPLEMENTATION
**********************************************************************/


inline long double RootInvestigator::F4(long double t, long double n) const {

    return 3 * pow(n*(t-1),2) + pow(n*(t-1),3) + 6 * t - n * (2 + 3*t -6 * pow(t,2)+pow(t,3));

}


inline RootInvestigator::RootInvestigator(long double lambda, long double length, int max_depth) {    

    t = 1/lambda;
    l = length;
    
    //The (approximate) depth at which the inequalities (5.2) become meaningless
    int MD = int(-log(l) / log(t))+1;   
    depth = min(min(MD,max_depth),MAX_DEPTH);
    
    //Set up the table of values corresponding to the RHS of formula (5.2).
    for (int n=0; n<MAX_DEPTH; n++) {
        compValue[n][MAX_DER-1] = maxDigit * ( l * f_derivativeBound(n,MAX_DER-1) + f_tailBound(n,MAX_DER-1) + F_P_ERROR );
        for (int j=MAX_DER-2; j >= 0; j--)
                compValue[n][j] = l * compValue[n][j+1] + maxDigit * (f_tailBound(n,j) + F_P_ERROR);
    }
        
}

//The left term of the expression in the RHS of displayed formula (5.2).
//This part arises as an upper bound for the tail of the hypothetical power series having a multiple zero.
//Notice that, referring again to (5.2), t = 1/b 
inline long double RootInvestigator::f_tailBound(int n, int j) const {

    switch(j) {
        case 0:
                return 1 / (t-1); 
        case 1:
                return ( t + n * t - n ) / pow((t-1), 2) ;
        case 2:
                return (  n * pow(t,2) + pow(n*(t-1),2) + 2 * t - n ) / pow (t-1, 3);
        case 3:
                return F4(t,n) / pow(t-1,4);
               
        default: 
                cout << "The case of " << j << " derivatives has not yet been implemented.";
    }

}

//The middle term on the RHS of (5.2), up to the length of the interval.
//This term arises as an upper bound coming for the application of the intermediate value theorem
//to the j-th derivative.
//Notice that this is only used for the highest order derivative; 
//then we inductively use the estimates obtained for the next higher derivative.
inline long double RootInvestigator::f_derivativeBound(int n,int j) const {

    return pow(t, n+2) / pow(t-1, j);

}

//The main procedure which runs the algorithm on the data supplied to the RootInvestigator object.
//The actual implementation is performed by the RootImplemenation object.
inline bool RootInvestigator::mayContainRoot() {

    return RootImplementation(this).mayContainRoot(depth);

}

inline AddressInfo RootInvestigator::getAddress() {

    ADDRESS addr;
    
    return RootImplementation(this).getAddress(depth, addr);

}

//The seed of the algorithm is the constant polyomial digit[0].
inline RootImplementation::RootImplementation(RootInvestigator* RI_) {

    RI = RI_;
    value[0] = digit[0];
    for (int j=1; j< MAX_DER; j++) 
        value[j] = 0;
    n = 0;
        

}

//Obtain the value of P(x) + digit_i x^(n+1) and its derivatives, evaluated at b,
//out of the values of P(x) and its derivatives. 
//In other words, here are gathered the values to be used in the LHS of (5.2).
inline RootImplementation::RootImplementation(RootImplementation& parent, SYMBOL nextDigit) {
    
    RI = parent.RI;
    n = parent.n+1;
    value[0] = parent.t() * parent.value[0] + digit[nextDigit];
    value[1] = parent.t() * parent.value[1] + (parent.n+1) * digit[nextDigit];
    for (int i=2; i<MAX_DER; i++)
        value[i] = parent.t() * parent.value[i] + C(parent.n+1,i) * digit[nextDigit];

}

inline RootImplementation RootImplementation::child(SYMBOL nextDigitIndex) {
    
    return RootImplementation(*this, nextDigitIndex);
}


//This procedure deals with a depth=0 case; i.e. it just checks whether all equations in (5.2) hold.
inline bool RootImplementation::mayContainRoot() {    

    for (int j=0; j < MAX_DER; j++) 
        if ( abs(value[j]) > (*RI).compValue[n][j] )
                return false;
                
    return true;

}

//This is the procedure where the main logic of the algorithm is defined.
bool RootImplementation::mayContainRoot(int depth) {

    if ( ! mayContainRoot() ) return false;   // There cannot be a zero

    /* if it wasn't possible to discard the possibility of a zero, there may be one */
    if (depth == 0) return true;                       
            
    for(int i=0; i< NDIGITS; i++)     
        if ( child(i).mayContainRoot(depth-1) )
                return true;
        

    /* None of the children reported that there could be a long double root, so there can't be one */
    return false;
    

}

//A version of the previous method which also keeps track of the address.
AddressInfo RootImplementation::getAddress(int depth, ADDRESS& l) {

    AddressInfo AI;

    if ( ! mayContainRoot() ) {
    
        AI.root = false;
        AI.addr = l;
        return AI;
    
    }
    
    if (depth == 0) {
        
        AI.root = true;
        AI.addr = l;
        return AI;
    
    }
    
    for(int i=0; i< NDIGITS; i++)  {
        AI = child(i).getAddress(depth - 1, l);
        if ( AI.root ) { 
                AI.addr.push_front(i);
                return AI;
        }
    }
    
    AI.root = false;
    AI.addr = l;
    return AI;

}


/*********************************************************************
                        GLOBAL PROCEDURES
**********************************************************************/

// The following procedure divides the interval (x,y) into mesh many subintervals
// and runs the algorithm on each (the depth is automatically determined).
// It outputs the result on both the standard output (i.e. the screen) and a file
// called intervals.txt (notice the contents of this file are reset on each run).
void checkInterval(long double x, long double y, int mesh) {


    ofstream out;
    out.open("intervals.txt");

    cout.precision(15);
    out.precision(15);
    
    bool check, DR;
    long double lastStart;
    long double step = (y-x)/mesh;    
    
    AddressInfo AI;
        
    DR = RootInvestigator(x+step, step, MAX_DEPTH).mayContainRoot();
    lastStart = x;
    
    for (long double i=x; i<y ; i+=step) {
        check = RootInvestigator(i+step, step, MAX_DEPTH).mayContainRoot();
        if ( (check != DR) || (i+step >= y) ) {
                cout << "[" << lastStart  << ", " << min(i+step,y) << "] : ";
                out << "[" << lastStart  << ", " << min(i+step,y) << "] : ";
                const char* str = DR ? "Possible root\n" : "No root\n";
                cout << str; out << str;
                DR = check;
                lastStart = i+step;
        }
    }
    
    out.close();

}

//This is an auxiliary procedure for the findIntervals method
int meshFromPrecision(double x, double y, long double e) {
        int d = 1;
        while ( 0.4 * (d+1) * pow(y, d+1) > e ) d++;
        return int((y-x) * pow(x,-d))+1;
}

//This procedure takes a desired precision e, and essentially finds the set of double roots
//in the interval (x,y) up to an error e (in other words, there is a double root within e
//of any point in the intervals returned by the algorithm). This is not rigorous. However,
//heuristically the results are correct and it is not hard to verify each case rigorously
//by using the polyomials obtained elsewhere in the program.
void findIntervals(long double x, long double y, long double e) { 

    ofstream out;
    out.open("intervals.txt");

    int prec = int(-log(e)/log(10.0)) + 2;
    cout.precision(prec);
    out.precision(prec);
    
    bool check, DR, firstInterval=true;
    long double lastStart;
    int mesh = meshFromPrecision(x,y,e);
    long double step = (y-x)/mesh;  
    
    AddressInfo AI;
        
    DR = RootInvestigator(x+step, step, MAX_DEPTH).mayContainRoot();
    lastStart = x;
    
    cout << "{ ";
    out << "{ ";
    
    for (long double i=x; i<y ; i+=step) {
        check = RootInvestigator(i+step, step, MAX_DEPTH).mayContainRoot();
        if ( (check != DR) || (i+step >= y) ) {
                if (!check) {
                   if (firstInterval)
                      firstInterval = false;
                   else
                   {
                      cout << ",";
                      out << ",";
                   }
                   cout << "{" << lastStart << "," << min(i+step,y) << "}";
                   out << "{" << lastStart << "," << min(i+step,y) << "}";

                }
                DR = check;
                lastStart = i+step;
        }
    }
    
    cout << " }\n";
    out << " }\n";
    
    out.close();

}

//If the algorithm does not die out in a certain interval, then it
//provides an address (polynomial) which comes very close to having a multiple zero.
//This procedure returns such a polynomial for the interval (lambda-delta,lambda).
//Note that if the algorithm dies before iter iterations this procedure returns
//a polynomial corresponding to the last iteration before it died.
//This polynomial can then be copied and pasted onto a mathematics package to check,
//for instance, if it is good for some interval.
//In practice, one uses a lambda extremely small.
void showAddress(long double lambda, long double delta, int iter) {
    
    ofstream out;
    out.open("address.txt");
    
    ADDRESS Ad;
    ADDRESS_IT AI;
    int i=1;
    long double curDigit;
    
    Ad = RootInvestigator(lambda,delta,iter).getAddress().addr;
    cout << "+" << digit[0];
    for (AI = Ad.begin(); AI != Ad.end(); AI++)
    {
        curDigit = digit[*AI];
        cout << "+(" << curDigit << ") x^" << i;
//       if (curDigit>0) { cout<< "+"; }
//       if (curDigit<0) { cout<< "-"; }  
//       if (curDigit==0) { cout<< "0"; }      
        i++;
    }   
    cout << "\n\n";
    
    out.close();

}


/*********************************************************************
                            EXAMPLES
**********************************************************************/

/* NOTE: These examples verify the assertions made in the paper regarding the double zeros of 
    power series with coefficients -1, 0, 1, corresponding to the settings
       NDIGITS=3 MAX_DER=2, digit[]={1,-1,0}, maxDigit=1 .
    The facts asserted in the last section can be verified analogously, recompiling the
    program after redefining:
       NDIGITS=5, MAX_DER=2, digit[]={1,-1,2,-2,0}, maxDigit=2
    and
       NDIGITS=3 MAX_DER=3, digit[]={1,-1,0}, maxDigit=1
    respectively.
*/

//Auxiliary functions

void readAnyChar() {
    char c;
    cout << "\nEnter any character to continue..."; cin >> c; cout << "\n";    
}

void outputAndCheckInterval(long double x, long double y, int m) {
    cout << "\nChecking interval (" << x << ", " << y << ") with a mesh of " << m << " subintervals\n";
    checkInterval(x,y,m);
    readAnyChar();
}

//This example shows how to obtain the lower bound 0.6684755 starting with no information at all

void exampleSmallestDRoot() {
    outputAndCheckInterval(0.5,0.7,1000); // .668 is the new lower bound
    outputAndCheckInterval(0.668,0.68,1000); // .668444 is the new lower bound
    outputAndCheckInterval(0.668444,0.67,5000); // .6684744976 is the new lower bound
    outputAndCheckInterval(0.6684744976, 0.6685, 20000); // .668475524... > 0.6684755 is the new lower bound
}

//This example proves that the supremum of the set of numbers which are NOT double roots is at least 0.669355\
//NOTE: This example assumes that NDIGITS = 3, MAX_DER = 2 and digit[] = { 1, -1, 0 }
void exampleLargestNonDRoot() {
    outputAndCheckInterval(0.669355, 0.669356, 100);
}

//This example proves that the intervals I_j given in the paper are indeed free of double roots
//NOTE: This example assumes that NDIGITS = 3, MAX_DER = 2 and digit[] = { 1, -1, 0 }
void exampleProveGaps() {
    outputAndCheckInterval(0.668478, .668489, 1000); // no roots here
    outputAndCheckInterval(0.668632, .668653, 1000); // no roots here 
    outputAndCheckInterval(0.669282, .669306, 2000); // no roots here   
    outputAndCheckInterval(0.669327, .669333, 1000); // no roots here
    outputAndCheckInterval(0.669343, .669352, 1000); // no roots here
    
    /* Remark: to prove that these are indeed (part of) different gaps we used the function 
        showAddress(...) to obtain the polynomials described in the paper */
}

//This example shows how to obtain the lower bound 0.5436 when the coefficient set is (-2,-1,0,1,2).
//NOTE: This example assumes that NDIGITS = 5, MAX_DER = 2 and digit[] = { 1, -1, 2, -2 0 }.


/*********************************************************************
                           MAIN ROUTINE
**********************************************************************/

int main() {                                                                       
                   
        //exampleSmallestDRoot();
        //exampleLargestNonDRoot();
        //exampleProveGaps();
                                                                                                                                                                                                                                                                                                                                                                               
        int option=0;
        
        long double x,y,lambda,delta;
        int mesh, iter;
        
        while (option!=4) {
             cout <<
              "\nEnter:" << 
                          "\n 1 to find a polynomial given an interval and depth;" <<
                          "\n 2 to run the algorithm on a given mesh of small intervals;" <<
                          "\n 3 to find the set of multiple roots in an interval up to a given precision;" <<
                          "\n 4 to quit...\n";
             cin >> option;
             switch(option){
                case 1:
                    cout << "\nEnter right point of interval: ";
                    cin >> lambda;
                    cout << "\nEnter length of interval: ";
                    cin >> delta;
                    cout << "\nEnter maximum number of iterations: ";
                    cin >> iter;
                    cout << "\n";                
                    showAddress(lambda, delta, iter);
                    break;
                case 2:
                    cout << "\nEnter left point: ";
                    cin >> x;
                    cout << "Enter right point: ";
                    cin >> y;
                    cout << "Enter number of subdivisions: ";
                    cin >> mesh;
                    
                    cout << "\n\nDepth is up to " << int(log((y-x)/mesh)/log(y)) << "\n\n";
         
                    checkInterval(x,y,mesh);
                    break;
                case 3:
                    double e;
                    cout << "\nEnter left point: ";
                    cin >> x;
                    cout << "Enter right point: ";
                    cin >> y;
                    cout << "\nEnter accuracy: ";
                    cin >> e;

                    cout << "\n\nDepth is up to " << int(log((y-x)/meshFromPrecision(x,y,e))/log(y)) << "\n\n";
         
                    findIntervals(x,y,e);  
                    break;            
                    
                default:
                    break;
                
             }
        }
   
        return 0;
        
}
